from typing import Any, List
import json
from nettowel.logging import log
from nettowel.exceptions import NettowelRestconfError

import requests
import urllib3

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)  # type: ignore

content_type = {
    "json": "application/yang-data+json",
    "xml": "application/yang-data+xml",
}
accept = {
    "json": "application/yang-data+json, application/yang-data.errors+json",
    "xml": "application/yang-data+xml, application/yang-data.errors+xml",
}


def send_request(
    method: str,
    url: str,
    username: str,
    password: str,
    data: str = None,
    verify: bool = True,
    send_xml: bool = False,
    return_xml: bool = False,
) -> Any:
    try:
        headers = {
            "Content-Type": content_type["xml"] if send_xml else content_type["json"],
            "Accept": accept["xml"] if return_xml else accept["json"],
        }
        response = requests.request(
            method=method,
            url=url,
            headers=headers,
            auth=(username, password),
            data=data,
            verify=verify,
        )
        response.raise_for_status()
        if response.status_code == 204:
            return "204 No Content"
        if response.status_code == 201:
            return "201 Created"
        if response.text == "":
            return "Empty Response"
        return response.json() if not return_xml else response.text
    except requests.exceptions.ConnectionError as exc:
        raise NettowelRestconfError(str(exc), None)
    except requests.exceptions.RequestException as exc:
        if exc.request:
            if not exc.response.text:
                response = None
            else:
                if return_xml:
                    return exc.response.text
                try:
                    response = exc.response.json()
                except json.decoder.JSONDecodeError:
                    response = exc.response.text
        else:
            response = None
        log.debug(response)
        raise NettowelRestconfError(str(exc), response)



from typing import Any
from nettowel._common import needs
from nettowel.logging import log

_module = "ttp"

try:
    from ttp import ttp

    log.debug("Successfully imported %s", _module)
    TTP_INSTALLED = True

except ImportError:
    log.warning("Failed to import %s", _module)
    TTP_INSTALLED = False


def render_template(
    data: str,
    template: str,
) -> Any:
    needs(TTP_INSTALLED, "TTP", _module)
    try:
        parser = ttp(data=data, template=template)
        parser.parse()
        results = parser.result(structure="flat_list")
    except Exception as exc:
        raise exc
    return results



from typing import Tuple
from click import style
import typer
from rich import print_json, print
from rich.markdown import Markdown
from rich.prompt import Prompt
from rich.panel import Panel
from rich.text import Text
from rich.highlighter import ReprHighlighter

from nettowel.cli._common import get_typer_app
from nettowel.exceptions import (
    NettowelException,
)

app = get_typer_app(help="Netmiko functions")


_highlight_words = [
    (
        [
            "Idle",
            "Down",
            "down",
            "errors",
            "error",
            "collisions",
            "drops",
            "failures",
            "failure",
        ],
        "dark_red",
    ),
    (["never"], "orange4"),
]


@app.command()
def cli(
    ctx: typer.Context,
    cmd: str = typer.Argument(..., help="CLI command to execute on device"),
    host: str = typer.Option(
        ..., help="Hostname or IP address", envvar="NETTOWEL_HOST"
    ),
    device_type: str = typer.Option(
        ..., help="Netmiko device type", envvar="NETTOWEL_DEVICE_TYPE"
    ),
    user: str = typer.Option(None, help="Username for login", envvar="NETTOWEL_USER"),
    password: str = typer.Option(
        None, help="Login password", envvar="NETTOWEL_PASSWORD"
    ),
    port: int = typer.Option(
        default=22, help="Connection Port", envvar="NETTOWEL_PORT"
    ),
    secret: str = typer.Option(
        default="", help="Enable secret", envvar="NETTOWEL_SECRET"
    ),
    use_textfsm: bool = typer.Option(
        False,
        "--use-textfsm",
        help="Use textFSM to get structured data",
        envvar="NETTOWEL_NETMIKO_TEXTFSM",
    ),
    use_ttp: bool = typer.Option(
        False,
        "--use-ttp",
        help="Use TTP to get structured data",
        envvar="NETTOWEL_NETMIKO_TTP",
    ),
    ttp_template: str = typer.Option(
        None,
        help="Use TTP Template. Use it with the `use_ttp` parameter",
        envvar="NETTOWEL_NETMIKO_TTP_TEMPLATE",
    ),
    use_genie: bool = typer.Option(
        False,
        "--use-genie",
        help="Use Genie to get structured data",
        envvar="NETTOWEL_NETMIKO_GENIE",
    ),
    ssh_config_file: str = typer.Option(
        None, help="SSH Config file", envvar="NETTOWEL_SSH_CONFIG"
    ),
    use_keys: bool = typer.Option(
        False,
        "--use-keys",
        help="Use provided SSH Key. Parameter `key_file` is used.",
        envvar="NETTOWEL_USE_KEY",
    ),
    key_file: str = typer.Option(
        None,
        help="SSH Key file. Use it with the `key_file` parameter",
        envvar="NETTOWEL_KEY_FILE",
    ),
    session_log: str = typer.Option(
        None, help="File to store session log", envvar="NETTOWEL_SESSION_LOG"
    ),
    json: bool = typer.Option(default=False, help="json output"),
    raw: bool = typer.Option(default=False, help="raw output"),
) -> None:
    from nettowel.netmiko import send_command

    try:
        if not any([user, ssh_config_file]):
            user = Prompt.ask("Enter username")
        if not any([password, use_keys, ssh_config_file]):
            password = Prompt.ask(f"Enter password for user {user}", password=True)
        result = send_command(
            cmd=cmd,
            device_type=device_type,
            host=host,
            username=user,
            password=password,
            port=port,
            secret=secret,
            use_textfsm=use_textfsm,
            use_ttp=use_ttp,
            ttp_template=ttp_template,
            use_genie=use_genie,
            ssh_config_file=ssh_config_file,
            use_keys=use_keys,
            key_file=key_file,
            session_log=session_log,
        )
        if json:
            print_json(data={"cmd": cmd, "result": result})
        elif raw:
            print(result)
        else:
            output = ReprHighlighter()(Text(result))
            for words, style in _highlight_words:
                output.highlight_words(words, style)
            print(
                Panel(
                    output,
                    title=f"[yellow][bold]{host}[/bold] {cmd}",
                    border_style="blue",
                )
            )
        raise typer.Exit(0)

    except NettowelException as exc:
        typer.echo(str(exc), err=True)
        raise typer.Exit(1)


@app.command()
def device_types(
    ctx: typer.Context,
    json: bool = typer.Option(default=False, help="json output"),
) -> None:
    from nettowel.netmiko import get_device_types

    try:
        types = get_device_types()
        if json:
            print_json(data=types)
        else:
            output = Markdown("\n".join([f"- {x}" for x in types]))
            print(output)
        raise typer.Exit(0)

    except NettowelException as exc:
        typer.echo(str(exc), err=True)
        raise typer.Exit(1)


@app.command()
def autodetect(
    ctx: typer.Context,
    host: str = typer.Argument(..., help="Hostname or IP address"),
    user: str = typer.Option(None, help="Username for login", envvar="NETTOWEL_USER"),
    password: str = typer.Option(
        None, help="Login password", envvar="NETTOWEL_PASSWORD"
    ),
    port: int = typer.Option(
        default=22, help="Connection Port", envvar="NETTOWEL_PORT"
    ),
    secret: str = typer.Option(
        default="", help="Enable secret", envvar="NETTOWEL_SECRET"
    ),
    ssh_config_file: str = typer.Option(
        None, help="SSH Config file", envvar="NETTOWEL_SSH_CONFIG"
    ),
    use_keys: bool = typer.Option(
        False,
        "--use-keys",
        help="Use provided SSH Key. Parameter `key_file` is used.",
        envvar="NETTOWEL_USE_KEY",
    ),
    key_file: str = typer.Option(
        None,
        help="SSH Key file. Use it with the `key_file` parameter",
        envvar="NETTOWEL_KEY_FILE",
    ),
    session_log: str = typer.Option(
        None, help="File to store session log", envvar="NETTOWEL_SESSION_LOG"
    ),
    json: bool = typer.Option(default=False, help="json output"),
) -> None:
    from nettowel.netmiko import autodetect as netmiko_autodetect

    try:
        if not any([user, ssh_config_file]):
            user = Prompt.ask("Enter username")
        if not any([password, use_keys, ssh_config_file]):
            password = Prompt.ask(f"Enter password for user {user}", password=True)
        result = netmiko_autodetect(
            host=host,
            username=user,
            password=password,
            port=port,
            secret=secret,
            ssh_config_file=ssh_config_file,
            use_keys=use_keys,
            key_file=key_file,
            session_log=session_log,
        )
        if json:
            print_json(data={"result": result})
        else:
            print(result)
        raise typer.Exit(0)

    except NettowelException as exc:
        typer.echo(str(exc), err=True)
        raise typer.Exit(1)


if __name__ == "__main__":
    app()



from typing import Any


class NettowelException(Exception):
    ...


class NettowelDependencyMissing(NettowelException):
    def __init__(
        self,
        package: str,
        extra_group: str,
        msg_format: str = "{package} is not installed. Install it with pip: pip install nettowel[{group}]",
    ) -> None:
        self.package = package
        self.extra_group = extra_group
        self.msg = msg_format.format(package=package, group=extra_group)
        super().__init__(self.msg)


class NettowelSyntaxError(NettowelException):
    ...


class NettowelTimeoutError(NettowelException):
    ...


class NettowelUsageError(NettowelException):
    ...


class NettowelRestconfError(NettowelException):
    def __init__(
        self,
        error_str: str,
        server_msg: Any,
    ) -> None:
        self.error_str = error_str
        self.server_msg = server_msg
        super().__init__(self.error_str)



import typer
from rich import print_json, print
from rich.panel import Panel
from nettowel.cli._common import get_typer_app, auto_complete_paths, read_yaml


app = get_typer_app(help="[EXPERIMENTAL] Pandas tools")


@app.command()
def flatten(
    ctx: typer.Context,
    data: typer.FileText = typer.Argument(
        ...,
        exists=True,
        file_okay=True,
        dir_okay=False,
        readable=True,
        resolve_path=True,
        allow_dash=True,
        help="JSON/YAML file to flatten.",
        autocompletion=auto_complete_paths,
    ),
    sep: str = typer.Option(default=".", help="separator"),
    json: bool = typer.Option(default=False, help="json output"),
    raw: bool = typer.Option(default=False, help="raw output"),
) -> None:
    from nettowel.pandas import normalize

    input_data = read_yaml(data)
    result = normalize(input_data, sep=sep)

    if json:
        print_json(data={"result": result})
    elif raw:
        print(result)
    else:
        print(Panel(result, title=f"[yellow]flatten", border_style="blue"))


if __name__ == "__main__":
    app()



import sys
import typer
from typing import Any, Dict, List
from rich import print_json, print

from rich.panel import Panel
from rich.columns import Columns
from rich.console import Group
from rich.syntax import Syntax
from rich.tree import Tree
from rich.scope import render_scope
from rich.json import JSON

from nettowel.exceptions import NettowelDependencyMissing, NettowelSyntaxError
from nettowel.cli._common import (
    get_typer_app,
    auto_complete_paths,
    read_yaml,
    read_text,
)

app = get_typer_app(help="Templating (Jinja2) functions")


def _variable_tree(data: Dict[str, Any]) -> Tree:
    root = Tree(":books: Variables", style="blue", guide_style="blue")

    types = {
        "boolean": ":keycap_10: boolean",
        "null": ":no_entry: null",
        "number": ":input_numbers: number",
        "string": ":newspaper: string",
        "error": "[red]Type unknown[/]",
    }

    def add(tree: Tree, data: Dict[str, Any], required: bool = True) -> Tree:
        property_type = data.get("type")
        name = data.get("title")
        text = (
            f"[blue]{name} [i](required)[/]"
            if required
            else f"[green]{name} [i](optional)[/]"
        )
        if property_type in ["string", "boolean", "number", "null"]:
            tree.add(Group(text, f'[yellow]{types[data.get("type", "error")]}[/]'))

        if "anyOf" in data:
            any_of = Tree("anyOf", style="yellow", guide_style="yellow")
            [
                any_of.add(types[x.get("type", "error")])
                for x in data.get("anyOf", dict())
            ]
            tree.add(Group(text, any_of))

        if property_type == "array":
            items = data.get("items", dict())
            if items.get("type") == "object":
                color = "blue" if required else "green"
                tree.add(
                    Group(
                        text,
                        add(
                            Tree(
                                ":open_book: object", style="yellow", guide_style=color
                            ),
                            items,
                        ),
                    )
                )
            else:
                any_of_array = items.get("anyOf")
                array = Tree("array (anyOf)", style="yellow", guide_style="yellow")
                [array.add(types[x.get("type", "error")]) for x in any_of_array]
                tree.add(Group(text, array))

        if property_type == "object":
            for property, value in data.get("properties", dict()).items():
                add(tree, value, property in data.get("required", list()))

        return tree

    for property, value in data.get("properties", dict()).items():
        add(root, value, property in data.get("required", list()))
    return root


@app.command()
def render(
    ctx: typer.Context,
    template_file_name: typer.FileText = typer.Argument(
        ...,
        exists=True,
        file_okay=True,
        dir_okay=False,
        readable=True,
        resolve_path=True,
        allow_dash=True,
        metavar="TEMPLATE",
        help="Jinja template file",
        autocompletion=auto_complete_paths,
    ),
    data_file_name: typer.FileText = typer.Argument(
        ...,
        exists=True,
        file_okay=True,
        dir_okay=False,
        readable=True,
        resolve_path=True,
        allow_dash=True,
        metavar="DATA",
        help="Data file in YAML or JSON",
        autocompletion=auto_complete_paths,
    ),
    trim_blocks: bool = typer.Option(
        False, "--trim-blocks", help="Remove first newline after a block"
    ),
    lstrip_blocks: bool = typer.Option(
        False,
        "--lstrip-blocks",
        help="Stripping leading spaces and tabs from the start of a line to a block",
    ),
    keep_trailing_newline: bool = typer.Option(
        False, "--keep-trailing-newline", help="Preserve the trailing newline"
    ),
    json: bool = typer.Option(False, "--json", help="JSON output"),
    raw: bool = typer.Option(False, "--raw", help="Raw result output"),
    only_result: bool = typer.Option(
        False, "--print-result-only", help="Only print the result"
    ),
    floating_panels: bool = typer.Option(
        False,
        "--floating-panels",
        help="Adjust the width of the panel to fit the content",
    ),
) -> None:
    from nettowel.jinja import render_template

    try:
        template_text = read_text(template_file_name)
        input_data = read_yaml(data_file_name)

        result = render_template(
            template=template_text,
            data=input_data,
            trim_blocks=trim_blocks,
            lstrip_blocks=lstrip_blocks,
            keep_trailing_newline=keep_trailing_newline,
        ).strip()

        if json:
            return_data = {
                "result": result,
                "template": template_text,
                "input_data": input_data,
            }
            if only_result:
                print_json(data={"result": result})
            else:
                print_json(data=return_data)
        elif raw:
            print(result)
        else:
            if floating_panels:
                code_width = (
                    max(
                        [len(x) for x in template_text.split("\n")]
                        + [len(x) for x in result.split("\n")]
                    )
                    + 6
                )
            else:
                code_width = None
            panels: List[Panel] = []
            if not only_result:
                data_output = JSON.from_data(input_data)
                template_output = Syntax(
                    template_text,
                    "jinja",
                    line_numbers=True,
                    indent_guides=True,
                    code_width=code_width,
                )
                panels.append(
                    Panel(
                        template_output, title="[yellow]Template", border_style="blue"
                    )
                )
                panels.append(
                    Panel(data_output, title="[yellow]Data", border_style="blue")
                )
            panels.append(
                Panel(
                    Syntax(
                        result,
                        "yaml",
                        line_numbers=True,
                        indent_guides=True,
                        code_width=code_width,
                    ),
                    border_style="blue",
                    title="[yellow]Result",
                )
            )
            if not only_result and floating_panels:
                height = (
                    max(
                        [
                            len(result.split("\n")),
                            len(data_output.text.plain.split("\n")),
                            len(template_text.split("\n")),
                        ]
                    )
                    + 2
                )
                for p in panels:
                    p.height = height
            print(Columns(panels, equal=True))
        raise typer.Exit(0)

    except NettowelDependencyMissing as exc:
        typer.echo(str(exc), err=True)
        raise typer.Exit(1)
    except NettowelSyntaxError as exc:
        typer.echo(exc, err=True)
        raise typer.Exit(2)


@app.command()
def validate(
    ctx: typer.Context,
    template_file_name: typer.FileText = typer.Argument(
        ...,
        exists=True,
        file_okay=True,
        dir_okay=False,
        readable=True,
        resolve_path=True,
        allow_dash=True,
        metavar="TEMPLATE",
        help="Jinja template file",
        autocompletion=auto_complete_paths,
    ),
    json: bool = typer.Option(default=False, help="json output"),
) -> None:
    from nettowel.jinja import validate_template

    try:
        template_text = read_text(template_file_name)
        result, data = validate_template(template_text)
        if json:
            data["valid"] = result
            print_json(data=data)
        else:
            if result:
                print(
                    Panel(
                        "Template is valid :white_heavy_check_mark:",
                        border_style="green",
                        expand=False,
                    )
                )
            else:
                pannel_group = Group(
                    Panel(
                        "Template is [red bold]not[/] valide :hankey:",
                        border_style="red",
                    ),
                    f'The validation resulted in the error "{data["message"]}" in the line number {data["lineno"]}.',
                    f'The error is probably in the following line or before or after it.\n\n{data["line"]}\n',
                    render_scope(data, title=str(data["message"])),
                )
                print(
                    Panel(pannel_group, border_style="red", title="ERROR", expand=False)
                )
        if not result:
            raise typer.Exit(1)
        raise typer.Exit(0)

    except NettowelDependencyMissing as exc:
        typer.echo(str(exc), err=True)
        raise typer.Exit(1)


@app.command()
def variables(
    ctx: typer.Context,
    template_file_name: typer.FileText = typer.Argument(
        ...,
        exists=True,
        file_okay=True,
        dir_okay=False,
        readable=True,
        resolve_path=True,
        allow_dash=True,
        metavar="TEMPLATE",
        help="Jinja template file",
        autocompletion=auto_complete_paths,
    ),
    json: bool = typer.Option(default=False, help="json output"),
) -> None:
    from nettowel.jinja import get_variables

    try:
        template_text = read_text(template_file_name)
        result = get_variables(template_text)
        if json:
            data = {"result": result}
            print_json(data=data)
        else:
            print(_variable_tree(result))
    except NettowelDependencyMissing as exc:
        typer.echo(str(exc), err=True)
        raise typer.Exit(1)
    except NettowelSyntaxError as exc:
        typer.echo(exc, err=True)
        raise typer.Exit(2)


if __name__ == "__main__":
    app()



import sys
import typer
from typing import List
from rich import print_json, print

from rich.json import JSON
from rich.panel import Panel
from rich.columns import Columns
from rich.syntax import Syntax

from nettowel.cli._common import get_typer_app, read_text, auto_complete_paths


app = get_typer_app(help="TTP templating functions")


@app.command()
def render(
    ctx: typer.Context,
    data_file_name: typer.FileText = typer.Argument(
        ...,
        exists=True,
        file_okay=True,
        dir_okay=False,
        readable=True,
        resolve_path=True,
        allow_dash=True,
        metavar="DATA",
        help="RAW data like CLI show command outputs or configuration snippets",
        autocompletion=auto_complete_paths,
    ),
    template_file_name: typer.FileText = typer.Argument(
        ...,
        exists=True,
        file_okay=True,
        dir_okay=False,
        readable=True,
        resolve_path=True,
        allow_dash=True,
        metavar="TEMPLATE",
        help="TTP template used to get structured data from the raw data",
        autocompletion=auto_complete_paths,
    ),
    json: bool = typer.Option(default=False, help="json output"),
    only_result: bool = typer.Option(False, "--print-result-only", help="Raw output"),
    floating_panels: bool = typer.Option(
        False,
        "--floating-panels",
        help="Adjust the width of the panel to fit the content",
    ),
) -> None:
    from nettowel.ttp import render_template

    try:
        template_text = read_text(template_file_name)
        input_data = read_text(data_file_name)

        result = render_template(template=template_text, data=input_data)
        if json:
            print_json(data=result)

        else:
            result_output = JSON.from_data(result)
            if floating_panels:
                code_width = (
                    max(
                        [len(x) for x in template_text.split("\n")]
                        + [len(x) for x in result_output.text.plain.split("\n")]
                    )
                    + 6
                )
            else:
                code_width = None
            panels: List[Panel] = []
            if not only_result:
                data_output = Syntax(
                    input_data,
                    "jinja",
                    line_numbers=True,
                    indent_guides=True,
                    code_width=code_width,
                )
                template_output = Syntax(
                    template_text,
                    "jinja",
                    line_numbers=True,
                    indent_guides=True,
                    code_width=code_width,
                )
                panels.append(
                    Panel(
                        template_output, title="[yellow]Template", border_style="blue"
                    )
                )
                panels.append(
                    Panel(data_output, title="[yellow]Data", border_style="blue")
                )
            panels.append(
                Panel(
                    result_output,
                    border_style="blue",
                    title="[yellow]Result",
                )
            )
            if not only_result and floating_panels:
                height = (
                    max(
                        [
                            len(result_output.text.plain.split("\n")),
                            len(input_data.split("\n")),
                            len(template_text.split("\n")),
                        ]
                    )
                    + 2
                )
                for p in panels:
                    p.height = height
            print(Columns(panels, equal=True))
        raise typer.Exit(0)

    except NotImplementedError as exc:
        typer.echo(exc)
        raise typer.Exit(1)


if __name__ == "__main__":
    app()



from typing import IO, Any
import ruamel.yaml


def load(f: IO[str]) -> Any:
    """YAML load data from stream

    Args:
        f (IO[str]): IO Stream with data

    Returns:
        Any: YAML Data
    """
    yml = ruamel.yaml.YAML(typ="safe")
    return yml.load(f)



import typer

from rich.panel import Panel
from rich.text import Text
from rich.columns import Columns
from rich.markdown import Markdown
from rich.console import Group
from rich import print

from nettowel import __version__ as version
from nettowel.cli._common import get_typer_app, auto_complete_paths
from nettowel.cli.ip import app as ipaddress_app
from nettowel.cli.jinja import app as jinja_app
from nettowel.cli.ttp import app as ttp_app
from nettowel.cli.textfsm import app as textfsm_app
from nettowel.cli.diff import app as diff_app
from nettowel.cli.yaml import app as yaml_app
from nettowel.cli.nornir import app as nornir_app
from nettowel.cli.napalm import app as napalm_app
from nettowel.cli.netmiko import app as netmiko_app
from nettowel.cli.scrapli import app as scrapli_app
from nettowel.cli.restconf import app as restconf_app
from nettowel.cli.pandas import app as pandas_app
from nettowel.cli.help import get_qrcode, HELP_MARKDOWN

app = get_typer_app(help="Awesome collection of network automation functions")

for subapp, name in [
    (ipaddress_app, "ipaddress"),
    (jinja_app, "jinja"),
    (ttp_app, "ttp"),
    (textfsm_app, "textFSM"),
    (diff_app, "diff"),
    (yaml_app, "yaml"),
    (nornir_app, "nornir"),
    (napalm_app, "napalm"),
    (netmiko_app, "netmiko"),
    (scrapli_app, "scrapli"),
    (restconf_app, "restconf"),
    (pandas_app, "pandas"),
]:
    app.add_typer(subapp, name=name)


@app.command()
def help(
    ctx: typer.Context,
) -> None:
    qrcodes = []
    for title, data in [
        ("GitHub NetTowel", "https://github.com/InfrastructureAsCode-ch/nettowel"),
        ("@InfraAsCode", "https://twitter.com/InfraAsCode/with_replies"),
        ("infrastructureascode.ch", "https://infrastructureascode.ch/?from=nettowel"),
    ]:
        qrcodes.append(
            Panel(
                get_qrcode(data),
                title=f"[yellow][link={data}]{title}[/link]",
                border_style="blue",
            )
        )
    qr_panels = Columns(qrcodes, equal=True)
    title_panel = Panel(
        Text(f"NetTowel {version}", style="b yellow", justify="center"),
        border_style="yellow",
    )
    markdown = Markdown(HELP_MARKDOWN)
    top = Panel(
        Group(title_panel, markdown, qr_panels),
        title="[blue]Help",
        expand=False,
        border_style="green",
    )
    print(top)


def run() -> None:
    app()


if __name__ == "__main__":
    run()



from io import StringIO
from qrcode import QRCode

from nettowel.logging import log

HELP_MARKDOWN = """
**NetTowel** is a collection of network automation functions. 
> This project is in an early stage and still under heavy development.

Use cases
- Demonstrations
- Troubleshooting
- Fun

"""


def get_qrcode(data: str) -> str:
    log.debug("Create QRcode %s", data)
    qr = QRCode()
    qr.add_data(data)
    out = StringIO()
    qr.print_ascii(out)
    out.seek(0)
    return out.read().rstrip("\n")



import logging
from rich.logging import RichHandler

FORMAT = "%(message)s"


def configure_logger(level: int) -> None:
    logging.basicConfig(
        level=level, format=FORMAT, datefmt="[%X]", handlers=[RichHandler()]
    )
    log.setLevel(level)


log = logging.getLogger("nettowel")
log.info("Hello, World!")



from typing import Any, Tuple, Dict, Union
from nettowel.exceptions import NettowelDependencyMissing, NettowelSyntaxError
from nettowel.logging import log
from nettowel._common import needs

_module = "jinja"

try:
    from jinja2 import Environment, Undefined, exceptions

    log.debug("Successfully imported %s", _module)
    JINJA_INSTALLED = True

except ImportError:
    log.warning("Failed to import %s", _module)
    JINJA_INSTALLED = False


def validate_template(template: str) -> Tuple[bool, Dict[str, Union[str, int, None]]]:
    """Validate jinja2 template

    Args:
        template (str): jinja2 template

    Returns:
        Tuple[bool, Dict[str, Union[str, int, None]]]: (True, {}) if template is valid. (False, {"message": ..., "lineno": ..., "line": ...})
    """
    needs(JINJA_INSTALLED, "Jinja2", _module)
    try:
        Environment().parse(template)
        return True, dict()
    except exceptions.TemplateSyntaxError as exc:
        try:
            line = template.splitlines()[exc.lineno - 1]
        except IndexError:
            line = ""

        result = {"message": exc.message, "lineno": exc.lineno, "line": line}

        return False, result


def render_template(
    template: str,
    data: Any,
    trim_blocks: bool = False,
    lstrip_blocks: bool = False,
    keep_trailing_newline: bool = False,
) -> str:
    """Rendering jinja2 template

    Args:
        template (str): Jinja template
        data (Any): Data to add to the rendering context. If data is not a dictionary it will be stored in one under the key 'data'
        trim_blocks (bool, optional): If this is set to True the first newline after a block is removed. Defaults to False.
        lstrip_blocks (bool, optional): If this is set to True leading spaces and tabs are stripped from the start of a line to a block. Defaults to False.
        keep_trailing_newline (bool, optional): Preserve the trailing newline when rendering templates. Defaults to False.

    Returns:
        str: Rendered template
    """
    needs(JINJA_INSTALLED, "Jinja2", _module)
    jinja_env = Environment(
        trim_blocks=trim_blocks,
        lstrip_blocks=lstrip_blocks,
        keep_trailing_newline=keep_trailing_newline,
        # extensions=[],
        undefined=Undefined,
    )
    if not isinstance(data, dict):
        data = {"data": data}

    try:
        return jinja_env.from_string(template).render(**data)
    except exceptions.TemplateSyntaxError as exc:
        raise NettowelSyntaxError(str(exc))


def get_variables(template: str) -> Any:
    """Get a list of variables with jinja2schema

    Args:
        template (str): Jinja template

    Raises:
        Exception: _description_

    Returns:
        Any: Return Dict with JSON Schema data
    """
    needs(JINJA_INSTALLED, "Jinja2", _module)
    try:
        from jinja2schema import infer, to_json_schema
    except ImportError:
        raise NettowelDependencyMissing("jinja2schema", _module)

    try:
        schema = infer(template)
    except exceptions.TemplateSyntaxError as exc:
        raise NettowelSyntaxError(str(exc))
    return to_json_schema(schema)



from typing import Union
import sys
import typer
from urllib.parse import quote
from rich import print_json, print
from rich.syntax import Syntax
from rich.prompt import Prompt
from rich.json import JSON
from rich.panel import Panel

from nettowel.cli._common import get_typer_app, auto_complete_paths
from nettowel.exceptions import (
    NettowelRestconfError,
)
from nettowel.restconf import send_request

app = get_typer_app(help="RESTCONF functions")


def _send_request(
    path: str,
    method: str,
    host: str,
    user: str,
    password: str,
    port: int,
    send_xml: bool,
    return_xml: bool,
    json: bool,
    raw: bool,
    verify: bool,
    data_file: typer.FileText = None,
) -> None:
    try:
        if not user:
            user = Prompt.ask("Enter username")
        if not password:
            password = Prompt.ask(f"Enter password for user {user}", password=True)

        if data_file:
            if data_file == "-":
                data = sys.stdin.read()
            else:
                with open(data_file) as f:  # type: ignore
                    data = f.read()
        else:
            data = None

        result = send_request(
            method=method,
            url=f"https://{host}:{port}/restconf/data/{path}",
            username=user,
            password=password,
            send_xml=send_xml,
            return_xml=return_xml,
            verify=verify,
            data=data,
        )
        if raw:
            print(result)
        elif json:
            print_json(data=result)
        else:
            if return_xml:
                output: Union[Syntax, JSON] = Syntax(
                    result,
                    "xml",
                    line_numbers=True,
                    indent_guides=True,
                )
            else:
                output = JSON.from_data(result)
            print(
                Panel(
                    output,
                    title=f"[yellow][bold]{method}[/bold] {path}",
                    border_style="blue",
                )
            )
        raise typer.Exit(0)
    except NettowelRestconfError as exc:
        typer.echo(str(exc), err=True)
        if exc.server_msg:
            typer.echo(exc.server_msg, err=True)
        raise typer.Exit(1)


@app.command()
def get(
    ctx: typer.Context,
    path: str = typer.Argument(
        ..., help="RESTCONF path. Example: Cisco-IOS-XE-native:native/hostname"
    ),
    host: str = typer.Option(
        ..., help="Hostname or IP address", envvar="NETTOWEL_HOST"
    ),
    user: str = typer.Option(None, help="Username for login", envvar="NETTOWEL_USER"),
    password: str = typer.Option(
        None, help="Login password", envvar="NETTOWEL_PASSWORD"
    ),
    port: int = typer.Option(
        default=443, help="Connection Port", envvar="NETTOWEL_RESTCONF_PORT"
    ),
    send_xml: bool = typer.Option(
        False,
        "--send-xml",
        help="Send XML instead of JSON",
    ),
    return_xml: bool = typer.Option(
        False,
        "--return-xml",
        help="Recieve XML instead of JSON",
    ),
    verify: bool = typer.Option(
        False,
        "--no-verify",
        help="Ignore SSL certificate verification",
        envvar="NETTOWEL_VERIFY",
    ),
    json: bool = typer.Option(default=False, help="json output"),
    raw: bool = typer.Option(default=False, help="raw output"),
) -> None:
    _send_request(
        path=path,
        method="GET",
        host=host,
        user=user,
        password=password,
        port=port,
        send_xml=send_xml,
        return_xml=return_xml,
        json=json,
        raw=raw,
        verify=verify,
    )


@app.command()
def delete(
    ctx: typer.Context,
    path: str = typer.Argument(
        ..., help="RESTCONF path. Example: Cisco-IOS-XE-native:native/hostname"
    ),
    host: str = typer.Option(
        ..., help="Hostname or IP address", envvar="NETTOWEL_HOST"
    ),
    user: str = typer.Option(None, help="Username for login", envvar="NETTOWEL_USER"),
    password: str = typer.Option(
        None, help="Login password", envvar="NETTOWEL_PASSWORD"
    ),
    port: int = typer.Option(
        default=443, help="Connection Port", envvar="NETTOWEL_RESTCONF_PORT"
    ),
    send_xml: bool = typer.Option(
        False,
        "--send-xml",
        help="Send XML instead of JSON",
    ),
    return_xml: bool = typer.Option(
        False,
        "--return-xml",
        help="Recieve XML instead of JSON",
    ),
    verify: bool = typer.Option(
        False,
        "--no-verify",
        help="Ignore SSL certificate verification",
        envvar="NETTOWEL_VERIFY",
    ),
    json: bool = typer.Option(default=False, help="json output"),
    raw: bool = typer.Option(default=False, help="raw output"),
) -> None:
    _send_request(
        path=path,
        method="DELETE",
        host=host,
        user=user,
        password=password,
        port=port,
        send_xml=send_xml,
        return_xml=return_xml,
        json=json,
        verify=verify,
        raw=raw,
    )


@app.command()
def post(
    ctx: typer.Context,
    path: str = typer.Argument(
        ..., help="RESTCONF path. Example: Cisco-IOS-XE-native:native/hostname"
    ),
    data_file: typer.FileText = typer.Argument(
        ...,
        exists=True,
        file_okay=True,
        dir_okay=False,
        readable=True,
        resolve_path=True,
        allow_dash=True,
        metavar="DATA",
        help="Data to send. Use '-' to read from stdin",
        autocompletion=auto_complete_paths,
    ),
    host: str = typer.Option(
        ..., help="Hostname or IP address", envvar="NETTOWEL_HOST"
    ),
    user: str = typer.Option(None, help="Username for login", envvar="NETTOWEL_USER"),
    password: str = typer.Option(
        None, help="Login password", envvar="NETTOWEL_PASSWORD"
    ),
    port: int = typer.Option(
        default=443, help="Connection Port", envvar="NETTOWEL_RESTCONF_PORT"
    ),
    send_xml: bool = typer.Option(
        False,
        "--send-xml",
        help="Send XML instead of JSON",
    ),
    return_xml: bool = typer.Option(
        False,
        "--return-xml",
        help="Recieve XML instead of JSON",
    ),
    verify: bool = typer.Option(
        False,
        "--no-verify",
        help="Ignore SSL certificate verification",
        envvar="NETTOWEL_VERIFY",
    ),
    json: bool = typer.Option(default=False, help="json output"),
    raw: bool = typer.Option(default=False, help="raw output"),
) -> None:
    _send_request(
        path=path,
        method="POST",
        host=host,
        user=user,
        password=password,
        port=port,
        send_xml=send_xml,
        return_xml=return_xml,
        json=json,
        raw=raw,
        verify=verify,
        data_file=data_file,
    )


@app.command()
def PUT(
    ctx: typer.Context,
    path: str = typer.Argument(
        ..., help="RESTCONF path. Example: Cisco-IOS-XE-native:native/hostname"
    ),
    data_file: typer.FileText = typer.Argument(
        ...,
        exists=True,
        file_okay=True,
        dir_okay=False,
        readable=True,
        resolve_path=True,
        allow_dash=True,
        metavar="DATA",
        help="Data to send. Use '-' to read from stdin",
        autocompletion=auto_complete_paths,
    ),
    host: str = typer.Option(
        ..., help="Hostname or IP address", envvar="NETTOWEL_HOST"
    ),
    user: str = typer.Option(None, help="Username for login", envvar="NETTOWEL_USER"),
    password: str = typer.Option(
        None, help="Login password", envvar="NETTOWEL_PASSWORD"
    ),
    port: int = typer.Option(
        default=443, help="Connection Port", envvar="NETTOWEL_RESTCONF_PORT"
    ),
    send_xml: bool = typer.Option(
        False,
        "--send-xml",
        help="Send XML instead of JSON",
    ),
    return_xml: bool = typer.Option(
        False,
        "--return-xml",
        help="Recieve XML instead of JSON",
    ),
    verify: bool = typer.Option(
        False,
        "--no-verify",
        help="Ignore SSL certificate verification",
        envvar="NETTOWEL_VERIFY",
    ),
    json: bool = typer.Option(default=False, help="json output"),
    raw: bool = typer.Option(default=False, help="raw output"),
) -> None:
    _send_request(
        path=path,
        method="PUT",
        host=host,
        user=user,
        password=password,
        port=port,
        send_xml=send_xml,
        return_xml=return_xml,
        json=json,
        raw=raw,
        verify=verify,
        data_file=data_file,
    )


@app.command()
def patch(
    ctx: typer.Context,
    path: str = typer.Argument(
        ..., help="RESTCONF path. Example: Cisco-IOS-XE-native:native/hostname"
    ),
    data_file: typer.FileText = typer.Argument(
        ...,
        exists=True,
        file_okay=True,
        dir_okay=False,
        readable=True,
        resolve_path=True,
        allow_dash=True,
        metavar="DATA",
        help="Data to send. Use '-' to read from stdin",
        autocompletion=auto_complete_paths,
    ),
    host: str = typer.Option(
        ..., help="Hostname or IP address", envvar="NETTOWEL_HOST"
    ),
    user: str = typer.Option(None, help="Username for login", envvar="NETTOWEL_USER"),
    password: str = typer.Option(
        None, help="Login password", envvar="NETTOWEL_PASSWORD"
    ),
    port: int = typer.Option(
        default=443, help="Connection Port", envvar="NETTOWEL_RESTCONF_PORT"
    ),
    send_xml: bool = typer.Option(
        False,
        "--send-xml",
        help="Send XML instead of JSON",
    ),
    return_xml: bool = typer.Option(
        False,
        "--return-xml",
        help="Recieve XML instead of JSON",
    ),
    verify: bool = typer.Option(
        False,
        "--no-verify",
        help="Ignore SSL certificate verification",
        envvar="NETTOWEL_VERIFY",
    ),
    json: bool = typer.Option(default=False, help="json output"),
    raw: bool = typer.Option(default=False, help="raw output"),
) -> None:
    _send_request(
        path=path,
        method="PATCH",
        host=host,
        user=user,
        password=password,
        port=port,
        send_xml=send_xml,
        return_xml=return_xml,
        json=json,
        raw=raw,
        verify=verify,
        data_file=data_file,
    )


@app.command()
def head(
    ctx: typer.Context,
    path: str = typer.Argument(
        ..., help="RESTCONF path. Example: Cisco-IOS-XE-native:native/hostname"
    ),
    host: str = typer.Option(
        ..., help="Hostname or IP address", envvar="NETTOWEL_HOST"
    ),
    user: str = typer.Option(None, help="Username for login", envvar="NETTOWEL_USER"),
    password: str = typer.Option(
        None, help="Login password", envvar="NETTOWEL_PASSWORD"
    ),
    port: int = typer.Option(
        default=443, help="Connection Port", envvar="NETTOWEL_RESTCONF_PORT"
    ),
    send_xml: bool = typer.Option(
        False,
        "--send-xml",
        help="Send XML instead of JSON",
    ),
    return_xml: bool = typer.Option(
        False,
        "--return-xml",
        help="Recieve XML instead of JSON",
    ),
    verify: bool = typer.Option(
        False,
        "--no-verify",
        help="Ignore SSL certificate verification",
        envvar="NETTOWEL_VERIFY",
    ),
    json: bool = typer.Option(default=False, help="json output"),
    raw: bool = typer.Option(default=False, help="raw output"),
) -> None:
    _send_request(
        path=path,
        method="HEAD",
        host=host,
        user=user,
        password=password,
        port=port,
        send_xml=send_xml,
        return_xml=return_xml,
        json=json,
        raw=raw,
        verify=verify,
    )


@app.command()
def endcode(
    ctx: typer.Context,
    text: str = typer.Argument(..., help="Text to encode / quote"),
    json: bool = typer.Option(default=False, help="json output"),
    raw: bool = typer.Option(default=False, help="raw output"),
) -> None:
    result = quote(text, safe="")
    if json:
        print_json(data={"result": result})
    elif raw:
        print(result)
    else:
        print(Panel(result, title=f"[yellow]{text}", border_style="blue"))


if __name__ == "__main__":
    app()



from typing import Any, List
from nettowel._common import needs
from nettowel.logging import log
from nettowel.exceptions import NettowelTimeoutError

_module = "netmiko"

try:
    from netmiko import ConnectHandler, SSHDetect
    from netmiko.ssh_dispatcher import CLASS_MAPPER_BASE as NETMIKO_DEVICE_TYPES
    from netmiko.base_connection import BaseConnection
    from netmiko.ssh_exception import NetmikoTimeoutException

    log.debug("Successfully imported %s", _module)
    NETMIKO_INSTALLED = True

except ImportError:
    log.warning("Failed to import %s", _module)
    NETMIKO_INSTALLED = False


def get_device_types() -> List[str]:
    needs(NETMIKO_INSTALLED, "Netmiko", _module)
    return list(NETMIKO_DEVICE_TYPES.keys())


def send_command(
    cmd: str,
    device_type: str,
    host: str,
    username: str = None,
    password: str = None,
    port: int = 22,
    secret: str = "",
    use_textfsm: bool = False,
    use_ttp: bool = False,
    ttp_template: str = None,
    use_genie: bool = False,
    ssh_config_file: str = None,
    use_keys: bool = False,
    key_file: str = None,
    session_log: str = None,
) -> str:
    try:
        device: BaseConnection = ConnectHandler(
            device_type=device_type,
            host=host,
            username=username,
            password=password,
            port=port,
            secret=secret,
            ssh_config_file=ssh_config_file,
            use_keys=use_keys,
            key_file=key_file,
            session_log=session_log,
        )
        if not device.check_enable_mode():
            device.enable()

        return str(
            device.send_command(
                cmd,
                use_textfsm=use_textfsm,
                use_genie=use_genie,
                use_ttp=use_ttp,
                ttp_template=ttp_template,
            )
        )
    except NetmikoTimeoutException as esc:
        raise NettowelTimeoutError(str(esc))


def autodetect(
    host: str,
    username: str = None,
    password: str = None,
    port: int = 22,
    secret: str = "",
    ssh_config_file: str = None,
    use_keys: bool = False,
    key_file: str = None,
    session_log: str = None,
) -> str:
    guesser = SSHDetect(
        device_type="autodetect",
        host=host,
        username=username,
        password=password,
        port=port,
        secret=secret,
        ssh_config_file=ssh_config_file,
        use_keys=use_keys,
        key_file=key_file,
        session_log=session_log,
    )
    return str(guesser.autodetect())



from nettowel.exceptions import NettowelDependencyMissing
from nettowel.logging import log


def needs(needs: bool, package: str, group: str) -> bool:
    if not needs:
        log.warning(
            "Package %s is not installed. It is part of the nettowel group %s",
            package,
            group,
        )
        raise NettowelDependencyMissing(package, group)
    log.debug("Package %s is installed", package)
    return True



from typing import Any
import io
from nettowel.logging import log
from nettowel._common import needs

_module = "pandas"

try:
    import pandas as pd

    log.debug("Successfully imported %s", _module)
    PANDA_INSTALLED = True

except ImportError:
    log.warning("Failed to import %s", _module)
    PANDA_INSTALLED = False


def normalize(input_data: Any, sep: str) -> str:
    """Normalize/flatten yaml or json data

    Args:
        data (any): Python datastructure
        sep (str): Separator

    Returns:
        str: Normalized data as csv
    """
    needs(PANDA_INSTALLED, "pandas", _module)
    df = pd.json_normalize(input_data, sep=sep)
    with io.StringIO() as f:
        df.to_csv(f)
        return f.getvalue()



import sys
import typer
from typing import Any, Dict, Generator, Optional, List
from dotenv import load_dotenv, find_dotenv
from pathlib import Path
from nettowel.yaml import load as yaml_load
from nettowel.cli.logging import configure_logger, log

VERBOSE_LEVEL = -1
DOTENV_LOADED = False


def _load_dotenv(dotenv: str) -> None:
    global DOTENV_LOADED
    if dotenv:
        log.debug("Loading dotenv file %s", dotenv)
        load_dotenv(dotenv)
    else:
        if DOTENV_LOADED:
            log.debug("Default dotenv file already loaded")
            return
        log.debug("Search in increasingly higher folders for dotenv file from cwd")
        dotenv_file = find_dotenv(usecwd=True)
        if dotenv_file:
            log.debug("Loading dotenv file %s", dotenv_file)
            load_dotenv(dotenv_file)
    DOTENV_LOADED = True


def _config_logging(verbose: int) -> None:
    global VERBOSE_LEVEL
    if VERBOSE_LEVEL == -1:
        VERBOSE_LEVEL = verbose
    elif verbose == 0:
        return
    else:
        VERBOSE_LEVEL += verbose
    level = 10 * (5 - max(0, min(VERBOSE_LEVEL, 4)))
    configure_logger(level=level)
    log.debug("Set logging level to %d", level)


def auto_complete_paths(incomplete: str) -> Generator[str, None, None]:
    incomplete_path = Path(incomplete)
    if incomplete_path.is_dir():
        glob = incomplete_path.glob("*")
    else:
        glob = incomplete_path.parent.glob(f"{incomplete_path.stem}*")
    for path in glob:
        if path.is_dir():
            yield f"{path}/"
        yield str(path)


def callback(
    dotenv: typer.FileText = typer.Option(
        None,
        exists=True,
        file_okay=True,
        dir_okay=False,
        readable=True,
        resolve_path=True,
        help=".env configuration file. If no file is specified, an attempt is made to search the .env file from cwd.",
        autocompletion=auto_complete_paths,
    ),
    verbose: int = typer.Option(
        0,
        "--verbose",
        "-v",
        count=True,
        help="Set de logging level. Default level is 'CRITICAL'",
    ),
) -> None:
    _config_logging(verbose)
    _load_dotenv(str(dotenv) if dotenv else "")


def get_typer_app(help: str) -> typer.Typer:
    return typer.Typer(help=help, callback=callback)


def get_members(obj: object, members: Optional[List[str]] = None) -> Dict[str, Any]:
    if members is None:
        members = [member for member in dir(obj) if not member.startswith("_")]
    return {x: getattr(obj, x) for x in members}


def cleanup_dict(data: Dict[str, Any]) -> Dict[str, Any]:
    return {k: v for k, v in data.items() if not callable(v)}


def read_text(data_file_name: typer.FileText) -> str:
    if data_file_name == "-":
        return sys.stdin.read()
    else:
        with open(data_file_name) as template_file:  # type: ignore
            text: str = template_file.read()
        return text


def read_yaml(data_file_name: typer.FileText) -> Any:
    if data_file_name == "-":
        return yaml_load(sys.stdin)
    else:
        with open(data_file_name) as data_file:  # type: ignore
            return yaml_load(data_file)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nettowel', 'nettowel.cli']

package_data = \
{'': ['*']}

install_requires = \
['python-dotenv>=0.20.0,<0.21.0',
 'qrcode>=7.3.1,<8.0.0',
 'requests>=2.27.1,<3.0.0',
 'rich>=12.0.1,<13.0.0',
 'ruamel.yaml>=0.17.21,<0.18.0',
 'typer>=0.4.1,<0.5.0']

extras_require = \
{'full': ['Jinja2>=3.0.3,<4.0.0',
          'ttp>=0.8.4,<0.9.0',
          'textfsm>=1.1.2,<2.0.0',
          'napalm>=3,<4',
          'netmiko>=4,<5',
          'scrapli>=2022.1.30,<2023.0.0',
          'nornir>=3.2.0,<4.0.0',
          'jinja2schema>=0.1.4,<0.2.0',
          'pandas>=1,<2'],
 'jinja': ['Jinja2>=3.0.3,<4.0.0', 'jinja2schema>=0.1.4,<0.2.0'],
 'napalm': ['napalm>=3,<4'],
 'netmiko': ['netmiko>=4,<5'],
 'pandas': ['pandas>=1,<2'],
 'scrapli': ['scrapli>=2022.1.30,<2023.0.0'],
 'textfsm': ['textfsm>=1.1.2,<2.0.0'],
 'ttp': ['ttp>=0.8.4,<0.9.0']}

entry_points = \
{'console_scripts': ['nettowel = nettowel.cli.main:run',
                     'nt = nettowel.cli.main:run']}

setup_kwargs = {
    'name': 'nettowel',
    'version': '0.4.2',
    'description': 'Network Automation Collection',
    'long_description': '# nettowel\nCollection of useful network automation functions \n\n> ⚠️ `nettowel` is under heavy construction and not production ready. Feedback is highly appreciated.\n\n\n## Install\n\nYou can install it directly from pypi\n\n```bash\npip install nettowel\n```\n\nTo reduce the dependencies the extra dependencies are grouped\n\nThe following groups are available (more details in the pyproject.toml):\n\n- full\n- jinja\n- ttp\n- textfsm\n- napalm\n- netmiko\n- scrapli\n- nornir\n- pandas\n\n```bash\npip install nettowel[jinja]\npip install nettowel[full]\n```\n\n## Install from source\n\n```\ngit clone ....\ncd nettowel\npoetry install\npoetry run nettowel --help\n```\n\n\n## Help and shell auto-completion\n\nThanks to the library [typer](https://typer.tiangolo.com/), `nettowel` comes with a nice help and autocompletion install\n\n![help](https://raw.githubusercontent.com/InfrastructureAsCode-ch/nettowel/main/imgs/help.png)\n\n\n## Features\n\nMany features are not implemented yet and many features will come.\n\n\n\n### Jinja2\n\n#### render\n\n![jinja rendering 1](https://raw.githubusercontent.com/InfrastructureAsCode-ch/nettowel/main/imgs/jinja-render-3.png)\n\n![jinja rendering 2](https://raw.githubusercontent.com/InfrastructureAsCode-ch/nettowel/main/imgs/jinja-render-1.png)\n\n#### validate\n\n![jinja validate](https://raw.githubusercontent.com/InfrastructureAsCode-ch/nettowel/main/imgs/jinja-validate.png)\n\n#### variables\n\n![jinja variables](https://raw.githubusercontent.com/InfrastructureAsCode-ch/nettowel/main/imgs/jinja-variables.png)\n\n\n### TTP\n\n#### render\n\n![ttp render](https://raw.githubusercontent.com/InfrastructureAsCode-ch/nettowel/main/imgs/ttp-render.png)\n\n### Netmiko\n\n#### cli\n\n![netmiko cli](https://raw.githubusercontent.com/InfrastructureAsCode-ch/nettowel/main/imgs/netmiko-cli.png)\n\n#### autodetect\n\n![netmiko autodetect](https://raw.githubusercontent.com/InfrastructureAsCode-ch/nettowel/main/imgs/netmiko-autodetect.png)\n\n#### device-types\n\n![netmiko device types](https://raw.githubusercontent.com/InfrastructureAsCode-ch/nettowel/main/imgs/netmiko-device-types.png)\n\n\n### RESTCONF\n\n#### get\n\n![restconf get](https://raw.githubusercontent.com/InfrastructureAsCode-ch/nettowel/main/imgs/restconf-get.png)\n\n#### patch, delete\n\n![restconf patch delete](https://raw.githubusercontent.com/InfrastructureAsCode-ch/nettowel/main/imgs/restconf-patch-delete.png)\n\n#### post, put\n\n![restconf post put](https://raw.githubusercontent.com/InfrastructureAsCode-ch/nettowel/main/imgs/restconf-post-put.png)\n\n### ipaddress\n\n#### ip-info\n\n![ip info](https://raw.githubusercontent.com/InfrastructureAsCode-ch/nettowel/main/imgs/ip-info.png)\n\n#### network-info\n\n![network info](https://raw.githubusercontent.com/InfrastructureAsCode-ch/nettowel/main/imgs/network-info.png)\n\n\n### Help\n\n![Help QRcode](https://raw.githubusercontent.com/InfrastructureAsCode-ch/nettowel/main/imgs/nettowel-help.png)\n\n\n### Settings\n\nA `dotenv` file can be used as a settings file. The file can also be provided with the option `--dotenv`.\n\n![environment settings](https://raw.githubusercontent.com/InfrastructureAsCode-ch/nettowel/main/imgs/env-settings.png)\n\n\n### Piping\n\n![piping](https://raw.githubusercontent.com/InfrastructureAsCode-ch/nettowel/main/imgs/piping.png)\n\n\n\n## Building CLI Docs\n\n**At the moment `typer-cli` is not ready for typer 0.4.0**\n\n```\ntyper nettowel/cli/main.py utils docs --name nettowel --output CLI.md\n```\n\n## Contributing\n\n### Run tests:\n\n```bash\nmake tests\n```\n\n\n### Bump version:\n\nSteps: patch, minor, major, prepatch, preminor, premajor, prerelease.\n\n```bash\nmake bump ARGS=patch\n```',
    'author': 'ubaumann',
    'author_email': 'github@m.ubaumann.ch',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/InfrastructureAsCode-ch/nettowel/tree',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import Union
import typer
from rich import inspect as rich_inspect
from rich import print_json

from nettowel.cli._common import get_members, cleanup_dict, get_typer_app

from ipaddress import ip_address, ip_network, IPv4Address, IPv6Address


app = get_typer_app(help="IP Address tools")


@app.command()
def ip_info(
    ctx: typer.Context,
    ip: str,
    json: bool = typer.Option(default=False, help="json output"),
) -> None:
    try:
        ip_obj: Union[IPv4Address, IPv6Address] = ip_address(ip)
        data = get_members(ip_obj)
        data.pop("packed")
        if json:
            print_json(data=cleanup_dict(data))
        else:
            rich_inspect(ip_obj)
        raise typer.Exit(0)

    except ValueError as exc:
        typer.echo(exc, err=True)
        raise typer.Exit(1)


@app.command()
def network_info(
    ctx: typer.Context,
    network: str,
    json: bool = typer.Option(default=False, help="json output"),
) -> None:
    try:
        net_obj = ip_network(network)
        data = get_members(net_obj)
        for key in ["broadcast_address", "hostmask", "netmask", "network_address"]:
            data[key] = str(data[key])

        if json:
            print_json(data=cleanup_dict(data))
        else:
            rich_inspect(net_obj)
        raise typer.Exit(0)

    except ValueError as exc:
        typer.echo(exc, err=True)
        raise typer.Exit(1)


if __name__ == "__main__":
    app()



import typer
from rich import inspect as rich_inspect
from rich import print_json

from nettowel.cli._common import get_typer_app


app = get_typer_app(help="(Deep)Diff functions")


@app.command()
def text_diff(
    ctx: typer.Context,
    # demo: str,
    json: bool = typer.Option(default=False, help="json output"),
) -> None:
    try:
        raise NotImplementedError("Not Implemented Yet")
        if json:
            print_json(data=None)
        else:
            rich_inspect(None)
        raise typer.Exit(0)

    except NotImplementedError as exc:
        typer.echo(exc)
        raise typer.Exit(1)


@app.command()
def data_diff(
    ctx: typer.Context,
    # demo: str,
    json: bool = typer.Option(default=False, help="json output"),
) -> None:
    try:
        raise NotImplementedError("Not Implemented Yet")
        if json:
            print_json(data=None)
        else:
            rich_inspect(None)
        raise typer.Exit(0)

    except NotImplementedError as exc:
        typer.echo(exc)
        raise typer.Exit(1)


if __name__ == "__main__":
    app()
